from psycopg2 import sql

from bsition.backend.postgres.utils import get_connection


def create_relations_tables():
    conn = get_connection()
    cur = conn.cursor()
    cur.execute(
        sql.SQL(
            """
            CREATE TABLE table_access (
                user_id INTEGER REFERENCES users(id),
                table_id INTEGER,
                access_type INTEGER CHECK (access_type IN (1, 2, 3)),
                PRIMARY KEY (user_id, table_id)
            )
            """
        )
    )
    cur.execute(
        sql.SQL(
            """
            CREATE TABLE doc_access (
                user_id INTEGER REFERENCES users(id),
                doc_id INTEGER,
                access_type INTEGER CHECK (access_type IN (1, 2, 3)),
                PRIMARY KEY (user_id, doc_id)
            )
            """
        )
    )
    cur.execute(
        sql.SQL(
            """
            CREATE TABLE public_docs (
                doc_id INTEGER PRIMARY KEY
            )
            """
        )
    )
    conn.commit()


def give_access_table(user_id, table_id, access_type):
    give_access(user_id, table_id, access_type, "table")


def give_access_doc(user_id, table_id, access_type):
    give_access(user_id, table_id, access_type, "doc")


def give_access(user_id, id, access_type, destination):
    if destination != "table" and destination != "doc":
        raise "Invalid access destination"

    conn = get_connection()
    cur = conn.cursor()
    cur.execute(
        sql.SQL(
            """
            INSERT INTO {destination_name} (user_id, {destination_id}, access_type)
            VALUES ({user_id}, {id}, {access_type})
            ON CONFLICT (user_id, {destination_id}) DO UPDATE
            SET access_type = {access_type}
        """
        ).format(
            user_id=sql.Literal(user_id),
            destination_name=sql.Identifier(destination + "_access"),
            destination_id=sql.Identifier(destination + "_id"),
            access_type=sql.Literal(access_type),
            id=sql.Literal(id),
        )
    )
    conn.commit()


def has_access_table(user_id, table_id):
    return has_access(user_id, table_id, "table")


def has_access_doc(user_id, table_id):
    return has_access(user_id, table_id, "doc")


def has_access(user_id, id, destination):
    if destination != "table" and destination != "doc":
        raise "Invalid access destination"
    conn = get_connection()
    cur = conn.cursor()
    cur.execute(
        sql.SQL(
            "SELECT access_type FROM {destination_access} WHERE user_id = {user_id} AND {destination_id} = {id}"
        ).format(
            user_id=sql.Literal(user_id),
            destination_access=sql.Identifier(destination + "_access"),
            destination_id=sql.Identifier(destination + "_id"),
            id=sql.Literal(id),
        )
    )
    return list(cur.fetchall())


def access_to_doc(id):
    conn = get_connection()
    cur = conn.cursor()
    cur.execute(
        sql.SQL(
            "SELECT user_id, username, access_type FROM doc_access JOIN users ON user_id = users.id WHERE doc_id = {id}"
        ).format(
            id=sql.Literal(id),
        )
    )
    return list(cur.fetchall())


def is_public(doc_id):
    conn = get_connection()
    cur = conn.cursor()
    cur.execute(
        sql.SQL("SELECT doc_id FROM public_docs WHERE doc_id = {doc_id}").format(
            doc_id=sql.Literal(doc_id),
        )
    )
    return cur.rowcount > 0


def make_public(doc_id):
    conn = get_connection()
    cur = conn.cursor()
    cur.execute(
        sql.SQL(
            "INSERT INTO public_docs VALUES {doc_id} ON CONFLICT DO NOTHING"
        ).format(
            doc_id=sql.Literal(doc_id),
        )
    )


def make_private(doc_id):
    conn = get_connection()
    cur = conn.cursor()
    cur.execute(
        sql.SQL(
            "DELETE FROM public_docs WHERE doc_id = {doc_id} ON CONFLICT DO NOTHING"
        ).format(
            doc_id=sql.Literal(doc_id),
        )
    )


def deny_access_table(user_id, table_id):
    return deny_access(user_id, table_id, "table")


def deny_access_doc(user_id, table_id):
    return deny_access(user_id, table_id, "doc")


def deny_access(user_id, id, destination):
    if destination != "table" and destination != "doc":
        raise "Invalid access destination"

    conn = get_connection()
    cur = conn.cursor()
    cur.execute(
        sql.SQL(
            "DELETE FROM {destination_access} WHERE user_id = {user_id} AND {destination_id} = {id}"
        ).format(
            user_id=sql.Literal(user_id),
            destination_access=sql.Identifier(destination + "_access"),
            destination_id=sql.Identifier(destination + "_id"),
            id=sql.Literal(id),
        )
    )
    conn.commit()


def get_accesible_documents(user_id):
    conn = get_connection()
    cur = conn.cursor()
    cur.execute(
        sql.SQL("SELECT doc_id FROM doc_access WHERE user_id = {user_id}").format(
            user_id=sql.Literal(user_id)
        )
    )

    docs = []
    for doc in cur.fetchall():
        docs.append(doc[0])
    return docs
