import io from "socket.io-client";
import {useState, useEffect} from "react";
import {useRouter} from "next/router";
import Layout from "../../components/Layout";
import {
    OptionsIcon,
    DeleteIcon,
    LockIcon,
    PublicIcon,
    ShareIcon,
    ManageIcon
} from "../../components/icons";
import Link from "next/link";

let socket;

export default function Document() {
    const router = useRouter();

    const [text, setText] = useState("");
    const [token, setToken] = useState("");
    const [title, setTitle] = useState("");

    useEffect(() => {
        socketInitializer();
    }, [router.query.id]);

    useEffect(() => {
        setToken(() => localStorage.getItem("token"))
    }, [])

    useEffect(() => {
        if (token && router.query.id) {
            const fetchData = async () => {
                const res = await fetch(`http://localhost:8000/api/documents/${router.query.id}`, {
                    method: "GET",
                    headers: {
                        'Authorization': `Bearer ${token}`
                    }
                })
                const json = await res.json();
                setText(() => json["data"])
                setTitle(() => json["name"])
            }

            fetchData()
        }
    }, [token, router.query.id])

    useEffect(() => {
        if (token && router.query.id) {
            const interval = setInterval(() => {
                fetch(`http://localhost:8000/api/documents/${router.query.id}`, {
                    method: 'PUT',
                    body: `{"data": "${text}"}`,
                    headers: {
                        'Content-type': 'application/json',
                        'Authorization': `Bearer ${token}`
                    }
                })
            }, 500);

            return () => clearInterval(interval);
        }
    }, [token, text, router.query.id])

    const socketInitializer = async () => {
        await fetch("/api/socket");

        socket = io();

        socket.emit('create', router.query.id);

        socket.on("newIncomingText", (text) => {
            setText(() => text);
            console.log('text' + text);
        });
    };

    const sendText = async () => {
        socket.emit("createdText", text, router.query.id);
        setText(() => text);
    };

    const handleKey = () => {
        if (text) {
            sendText();
        }
    };

    let access = router.query.id + "/access"

    return (
        <Layout>
            <div style={{height: "50px", justifyContent: "space-between"}}
                 className="align-center flex">
              <span className="docTitle">
                  {title}
              </span>
                <div className="dropdown">
                    <OptionsIcon onClick={(e) => {
                        document.getElementById("myDropdown").classList.toggle("show");

                        window.onclick = function (event) {
                            if (!event.target.matches('.dropbtn')) {
                                var dropdowns = document.getElementsByClassName("dropdown-content");
                                var i;
                                for (i = 0; i < dropdowns.length; i++) {
                                    var openDropdown = dropdowns[i];
                                    if (openDropdown.classList.contains('show')) {
                                        openDropdown.classList.remove('show');
                                    }
                                }
                            }
                        }
                    }} className="dropbtn"></OptionsIcon>
                    <div id="myDropdown" className="dropdown-content">
                        <Link href={access}>
                            <div className="flex flex-row items-center justify-start align-center menu-opt">
                                <ManageIcon/><span>Manage access</span></div>
                        </Link>
                        <a href="#">
                            <div className="flex flex-row items-center justify-start align-center menu-opt"><ShareIcon/><span>Share</span>
                            </div>
                        </a>
                        <a href="#">
                            <div className="flex flex-row items-center justify-start align-center menu-opt">
                                <LockIcon/><span>Public/private</span></div>
                        </a>
                        <a href="#">
                            <div className="flex flex-row items-center justify-start align-center menu-opt">
                                <DeleteIcon/><span>Delete document</span></div>
                        </a>
                    </div>
                </div>
            </div>
            <textarea
                className="document_area"
                placeholder="Type..."
                onKeyUp={handleKey}
                value={text}
                onChange={(e) => setText(e.target.value)}
            />
        </Layout>
    );
}